@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,6 +51,7 @@ package com.openexchange.data.conversion.ical.ical4j;
 
 import java.io.BufferedReader;
 import java.io.ByteArrayInputStream;
+import java.io.Closeable;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
@@ -61,13 +62,17 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.LinkedList;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
 import java.util.TimeZone;
 import net.fortuna.ical4j.data.CalendarBuilder;
 import net.fortuna.ical4j.data.ParserException;
 import net.fortuna.ical4j.model.Component;
+import net.fortuna.ical4j.model.Parameter;
+import net.fortuna.ical4j.model.Property;
 import net.fortuna.ical4j.model.component.CalendarComponent;
 import net.fortuna.ical4j.model.component.VEvent;
+import net.fortuna.ical4j.model.component.VFreeBusy;
 import net.fortuna.ical4j.model.component.VToDo;
 import net.fortuna.ical4j.model.property.Completed;
 import net.fortuna.ical4j.model.property.DateProperty;
@@ -79,34 +84,41 @@ import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import com.openexchange.data.conversion.ical.ConversionError;
 import com.openexchange.data.conversion.ical.ConversionWarning;
+import com.openexchange.data.conversion.ical.FreeBusyInformation;
 import com.openexchange.data.conversion.ical.ICalParser;
 import com.openexchange.data.conversion.ical.ical4j.internal.AppointmentConverters;
 import com.openexchange.data.conversion.ical.ical4j.internal.AttributeConverter;
+import com.openexchange.data.conversion.ical.ical4j.internal.FreeBusyConverters;
+import com.openexchange.data.conversion.ical.ical4j.internal.ParserTools;
 import com.openexchange.data.conversion.ical.ical4j.internal.TaskConverters;
 import com.openexchange.groupware.calendar.CalendarDataObject;
 import com.openexchange.groupware.container.Appointment;
 import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.tasks.Task;
 
+import edu.emory.mathcs.backport.java.util.Arrays;
+
 /**
+ * {@link ICal4JParser} - The {@link ICalParser} using <a href="http://ical4j.sourceforge.net/">ICal4j</a> library.
+ *
  * @author Francisco Laguna <francisco.laguna@open-xchange.com>
- * @author Tobias Prinz <tobias.prinz@open-xchange.com> (bug 11958, 16367)
+ * @author Tobias Prinz <tobias.prinz@open-xchange.com> (bug workarounds)
  */
 public class ICal4JParser implements ICalParser {
 
     private static final String UTF8 = "UTF-8";
 
-    private static final Log LOG = LogFactory.getLog(ICal4JParser.class);
+    private static final Log LOG = com.openexchange.log.Log.valueOf(LogFactory.getLog(ICal4JParser.class));
 
-    private static final Map<String, Integer> weekdays = new HashMap<String, Integer>();
+    private static final Map<String, Integer> WEEKDAYS = new HashMap<String, Integer>(7);
     static {
-        weekdays.put("MO", Integer.valueOf(Appointment.MONDAY));
-        weekdays.put("TU", Integer.valueOf(Appointment.TUESDAY));
-        weekdays.put("WE", Integer.valueOf(Appointment.WEDNESDAY));
-        weekdays.put("TH", Integer.valueOf(Appointment.THURSDAY));
-        weekdays.put("FR", Integer.valueOf(Appointment.FRIDAY));
-        weekdays.put("SA", Integer.valueOf(Appointment.SATURDAY));
-        weekdays.put("SO", Integer.valueOf(Appointment.SUNDAY));
+        WEEKDAYS.put("MO", Integer.valueOf(Appointment.MONDAY));
+        WEEKDAYS.put("TU", Integer.valueOf(Appointment.TUESDAY));
+        WEEKDAYS.put("WE", Integer.valueOf(Appointment.WEDNESDAY));
+        WEEKDAYS.put("TH", Integer.valueOf(Appointment.THURSDAY));
+        WEEKDAYS.put("FR", Integer.valueOf(Appointment.FRIDAY));
+        WEEKDAYS.put("SA", Integer.valueOf(Appointment.SATURDAY));
+        WEEKDAYS.put("SO", Integer.valueOf(Appointment.SUNDAY));
     }
 
     public ICal4JParser() {
@@ -115,7 +127,11 @@ public class ICal4JParser implements ICalParser {
         CompatibilityHints.setHintEnabled(
                 CompatibilityHints.KEY_OUTLOOK_COMPATIBILITY, true);
         CompatibilityHints.setHintEnabled(
-                        CompatibilityHints.KEY_NOTES_COMPATIBILITY, true);
+                CompatibilityHints.KEY_NOTES_COMPATIBILITY, true);
+        CompatibilityHints.setHintEnabled(
+        		CompatibilityHints.KEY_RELAXED_PARSING, true);
+        CompatibilityHints.setHintEnabled(
+              	CompatibilityHints.KEY_RELAXED_VALIDATION, true);
 
     }
 
@@ -147,9 +163,13 @@ public class ICal4JParser implements ICalParser {
                     }
                 }
             }
-            
+
         } catch (final UnsupportedEncodingException e) {
             // IGNORE
+        } catch (final ConversionError e){
+        	errors.add(e);
+        } finally {
+            closeSafe(reader);
         }
 
 
@@ -157,6 +177,84 @@ public class ICal4JParser implements ICalParser {
         return appointments;
     }
 
+	public List<FreeBusyInformation> parseFreeBusy(String icalText, TimeZone defaultTZ, Context ctx, List<ConversionError> errors, List<ConversionWarning> warnings) throws ConversionError {
+        try {
+            return parseFreeBusy(new ByteArrayInputStream(icalText.getBytes(UTF8)), defaultTZ, ctx, errors, warnings);
+        } catch (UnsupportedEncodingException e) {
+            LOG.error(e.getMessage(), e);
+        }
+        return Collections.emptyList();
+	}
+
+	public List<FreeBusyInformation> parseFreeBusy(InputStream ical, TimeZone defaultTZ, Context ctx, List<ConversionError> errors, List<ConversionWarning> warnings) throws ConversionError {
+        List<FreeBusyInformation> fbInfos = new ArrayList<FreeBusyInformation>();
+        BufferedReader reader = null;
+        try {
+            reader = new BufferedReader(new InputStreamReader(ical, UTF8));
+            while (true) {
+                net.fortuna.ical4j.model.Calendar calendar = parse(reader);
+                if (null == calendar) { 
+                	break; 
+                }
+                int i = 0;
+                for (Object componentObj : calendar.getComponents("VFREEBUSY")) {
+                    Component vevent = (Component) componentObj;
+                    try {
+                        fbInfos.add(convertFreeBusy(i++, (VFreeBusy)vevent, defaultTZ, ctx, warnings));
+                    } catch (ConversionError conversionError) {
+                        errors.add(conversionError);
+                    }
+                }
+            }
+        } catch (UnsupportedEncodingException e) {
+            LOG.error(e.getMessage(), e);
+            // IGNORE
+        } catch (ConversionError e) {
+        	errors.add(e);
+        } finally {
+            closeSafe(reader);
+        }
+        return fbInfos;
+	}
+	
+	private FreeBusyInformation convertFreeBusy(int index, VFreeBusy freeBusy, TimeZone defaultTZ, Context ctx, List<ConversionWarning> warnings) throws ConversionError {
+        FreeBusyInformation fbInfo = new FreeBusyInformation();
+        TimeZone tz = determineTimeZone(freeBusy, defaultTZ);
+        for (AttributeConverter<VFreeBusy, FreeBusyInformation> converter : FreeBusyConverters.REQUEST) {
+            if (converter.hasProperty(freeBusy)) {
+				converter.parse(index, freeBusy, fbInfo, tz, ctx, warnings);
+            }
+        }
+        fbInfo.setTimezone(getTimeZoneID(tz));
+        return fbInfo;
+
+	}
+
+    public String parseProperty(final String propertyName, final InputStream ical) {
+        if (null == propertyName || null == ical) {
+            return null;
+        }
+        BufferedReader reader = null;
+        try {
+            reader = new BufferedReader(new InputStreamReader(ical, UTF8));
+            final net.fortuna.ical4j.model.Calendar calendar = parse(reader);
+            if (calendar == null) {
+                return null;
+            }
+            final Property property = calendar.getProperty(propertyName.toUpperCase(Locale.US));
+            return null == property ? null : property.getValue();
+        } catch (final UnsupportedEncodingException e) {
+            // IGNORE
+            return null;
+        } catch (final ConversionError e){
+            return null;
+        } catch (final RuntimeException e){
+            return null;
+        } finally {
+            closeSafe(reader);
+        }
+    }
+
     public List<Task> parseTasks(final String icalText, final TimeZone defaultTZ, final Context ctx, final List<ConversionError> errors, final List<ConversionWarning> warnings) throws ConversionError {
         try {
             return parseTasks(new ByteArrayInputStream(icalText.getBytes(UTF8)), defaultTZ, ctx, errors, warnings);
@@ -168,8 +266,9 @@ public class ICal4JParser implements ICalParser {
 
     public List<Task> parseTasks(final InputStream ical, final TimeZone defaultTZ, final Context ctx, final List<ConversionError> errors, final List<ConversionWarning> warnings) throws ConversionError {
         final List<Task> tasks = new ArrayList<Task>();
+        BufferedReader reader = null;
         try {
-            final BufferedReader reader = new BufferedReader(new InputStreamReader(ical, UTF8));
+            reader = new BufferedReader(new InputStreamReader(ical, UTF8));
             while(true) {
                 final net.fortuna.ical4j.model.Calendar calendar = parse(reader);
                 if(calendar == null) { break; }
@@ -186,33 +285,45 @@ public class ICal4JParser implements ICalParser {
 
         } catch (final UnsupportedEncodingException e) {
             // IGNORE
+        } finally {
+            closeSafe(reader);
         }
-
-
         return tasks;
     }
 
-
-    protected CalendarDataObject convertAppointment(final int index, final VEvent vevent, final TimeZone defaultTZ, final Context ctx, final List<ConversionWarning> warnings) throws ConversionError {
+    protected static void closeSafe(final Closeable closeable) {
+        if (closeable != null) {
+            try {
+                closeable.close();
+            } catch (final IOException e) {
+                // Ignore
+            }
+        }
+    }
+    
+    protected CalendarDataObject convertAppointment(int index, VEvent vevent, TimeZone defaultTZ, List<AttributeConverter<VEvent,Appointment>> all, Context ctx, List<ConversionWarning> warnings) throws ConversionError {
 
         final CalendarDataObject appointment = new CalendarDataObject();
 
         final TimeZone tz = determineTimeZone(vevent, defaultTZ);
 
-        for (final AttributeConverter<VEvent, Appointment> converter : AppointmentConverters.ALL) {
+        for (final AttributeConverter<VEvent, Appointment> converter : all) {
             if (converter.hasProperty(vevent)) {
                 converter.parse(index, vevent, appointment, tz, ctx, warnings);
             }
             converter.verify(index, appointment, warnings);
         }
 
-        
-        
-        appointment.setTimezone(getTimeZoneID(tz));
+        appointment.setTimezone(getTimeZoneID(defaultTZ));
 
         return appointment;
     }
 
+
+    protected CalendarDataObject convertAppointment(final int index, final VEvent vevent, final TimeZone defaultTZ, final Context ctx, final List<ConversionWarning> warnings) throws ConversionError {
+        return convertAppointment(index, vevent, defaultTZ, AppointmentConverters.ALL, ctx, warnings);
+    }
+
     protected Task convertTask(final int index, final VToDo vtodo, final TimeZone defaultTZ, final Context ctx, final List<ConversionWarning> warnings) throws ConversionError{
         final TimeZone tz = determineTimeZone(vtodo, defaultTZ);
         final Task task = new Task();
@@ -235,22 +346,34 @@ public class ICal4JParser implements ICalParser {
             }
         }
 
-        return null;
+        return defaultTZ;
     }
 
     private static final TimeZone chooseTimeZone(final DateProperty dateProperty, final TimeZone defaultTZ) {
         TimeZone tz = defaultTZ;
-        if (dateProperty.isUtc()) {
-            tz = TimeZone.getTimeZone("UTC");
+        if (dateProperty.getParameter("TZID") == null 
+        || dateProperty.getParameter("TZID").getValue() == null){ 
+                return defaultTZ;
         }
-        final TimeZone inTZID = (null != dateProperty.getParameter("TZID")) ? TimeZone.getTimeZone(dateProperty.getParameter("TZID").getValue()) : null;
+        if(dateProperty.isUtc()) {
+        	tz = TimeZone.getTimeZone("UTC");
+        }
+        Parameter tzid = dateProperty.getParameter("TZID");
+		String tzidName = tzid.getValue();
+		TimeZone inTZID = (null != tzid) ? TimeZone.getTimeZone(tzidName) : null;
+		
+        /* now, if the Java core devs had been smart, they'd made TimeZone.getTimeZone(name,fallback) public. But they did not, so have to do this: */
+		if(inTZID.getID().equals("GMT") && ! tzidName.equals("GMT")){
+			inTZID = ParserTools.findTzidBySimilarity(tzidName);
+		}
+		
         if (null != inTZID) {
             tz = inTZID;
         }
         return tz;
     }
 
-    private String getTimeZoneID(final TimeZone tz) {
+	private String getTimeZoneID(final TimeZone tz) {
         if(net.fortuna.ical4j.model.TimeZone.class.isAssignableFrom(tz.getClass())) {
             return "UTC";
         }
@@ -260,20 +383,23 @@ public class ICal4JParser implements ICalParser {
         return tz.getID();
     }
 
-    private net.fortuna.ical4j.model.Calendar parse(final BufferedReader reader) throws ConversionError {
+    protected net.fortuna.ical4j.model.Calendar parse(final BufferedReader reader) throws ConversionError {
         final CalendarBuilder builder = new CalendarBuilder();
 
         try {
             final StringBuilder chunk = new StringBuilder();
             String line;
             boolean read = false;
-            boolean timezoneStarted = false; //hack to fix bug 11958 
+            boolean timezoneStarted = false; //hack to fix bug 11958
             boolean timezoneEnded = false; //hack to fix bug 11958
             boolean timezoneRead = false; //hack to fix bug 11958
             final StringBuilder timezoneInfo = new StringBuilder(); //hack to fix bug 11958
             // Copy until we find an END:VCALENDAR
             boolean beginFound = false;
             while((line = reader.readLine()) != null) {
+            	if(!beginFound && line.endsWith("BEGIN:VCALENDAR")){
+            		line = removeByteOrderMarks(line);
+            	}
                 if(line.startsWith("BEGIN:VCALENDAR")) {
                     beginFound = true;
                 } else if ( !beginFound && !"".equals(line)) {
@@ -310,10 +436,18 @@ public class ICal4JParser implements ICalParser {
             		}
             	}
             }
-            StringReader chunkedReader = new StringReader(
-                workaroundFor16613(
-                    workaroundFor16367(chunk.toString()))); // FIXME: Encoding!
-            return builder.build(chunkedReader); 
+            final StringReader chunkedReader = new StringReader(
+            	workaroundFor19463(
+            	workaroundFor16895(
+            	workaroundFor16613(
+            	workaroundFor16367(
+            	workaroundFor17492(
+            	workaroundFor17963(
+            	workaroundFor20453(
+            	removeAnnoyingWhitespaces(chunk.toString()
+                ))))))))
+            ); // FIXME: Encoding?
+            return builder.build(chunkedReader);
         } catch (final IOException e) {
             //IGNORE
         } catch (final ParserException e) {
@@ -323,15 +457,54 @@ public class ICal4JParser implements ICalParser {
         return null;
     }
 
-   
-    private String workaroundFor16367(String input) {
+    private String workaroundFor17963(final String input) {
+    	return input.replaceAll("EXDATE:(\\d+)([\\n\\r])", "EXDATE:$1T000000$2");
+	}
+
+	private String workaroundFor17492(final String input) {
+    	return input.replaceAll(";SCHEDULE-AGENT=", ";X-CALDAV-SCHEDULE-AGENT=");
+	}
+
+	private String workaroundFor19463(final String input) {
+		return input
+			.replaceAll("TZOFFSETFROM:\\s*(\\d\\d\\d\\d)", "TZOFFSETFROM:+$1")
+			.replaceAll("TZOFFSETTO:\\s*(\\d\\d\\d\\d)",   "TZOFFSETTO:+$1")
+			;
+	}
+	
+	private String workaroundFor20453(final String input) {
+		return input
+			.replaceAll("DTEND;\\s*\n", "")
+			;
+	}
+
+	/**
+     * Method written out of laziness: Because you can spread iCal attributes
+     * over several lines with newlines followed by a white space while a normal
+     * newline means a new attribute starts, one would need to parse the whole file
+     * (with a lookahead) before fixing errors. That means no regular expressions
+     * allowed. Since spreading just makes it nicer to read for humans, this method
+     * strips those newline+whitespace elements so we can use simple regexps.
+     */
+    private String removeAnnoyingWhitespaces(final String input) {
+		return input.replaceAll("\n\\s+", "");
+	}
+
+	private String workaroundFor16895(final String input) {
+		/* Bug in Zimbra: They like to use an EMAIL element for the
+		 * ATTENDEE property, though there is none.
+		 */
+		return input.replaceAll("ATTENDEE([^\n]*?);EMAIL=", "ATTENDEE$1;X-ZIMBRA-EMAIL=");
+	}
+
+	private String workaroundFor16367(final String input) {
         /* Bug in MS Exchange: If you use a CN element, it must have a value.
-         * MS Exchange has an empty value, which we now replace properly. 
+         * MS Exchange has an empty value, which we now replace properly.
          */
         return input.replaceAll("CN=:", "CN=\"\":");
     }
-    
-    private String workaroundFor16613(String input) {
+
+    private String workaroundFor16613(final String input) {
         /*
          * Bug in Groupwise: There is no attribute ID for ATTACH. Experimental
          * ones are allowed, but they would start with X-GW for Groupwise.
@@ -340,4 +513,26 @@ public class ICal4JParser implements ICalParser {
         return input.replaceAll("\nATTACH(.*?);ID=(.+?)([:;])" , "\nATTACH$1$3");
     }
 
-}
\ No newline at end of file
+    private String removeByteOrderMarks(String line){
+    	char[] buf = line.toCharArray();
+    	int length = buf.length;
+
+		if(length > 3)
+			if(Character.getNumericValue(buf[0]) < 0 && Character.getNumericValue(buf[1]) < 0 && Character.getNumericValue(buf[2]) < 0 && Character.getNumericValue(buf[3]) < 0){
+				if(Character.getType(buf[0]) == 15 && Character.getType(buf[1]) == 15 && Character.getType(buf[2]) == 28 && Character.getType(buf[3]) == 28)
+					return new String(Arrays.copyOfRange(buf, 3, length));
+				if(Character.getType(buf[0]) == 28 && Character.getType(buf[1]) == 28 && Character.getType(buf[2]) == 15 && Character.getType(buf[3]) == 15)
+					return new String(Arrays.copyOfRange(buf, 3, length));
+			}
+		if(length > 1)
+			if(Character.getNumericValue(buf[0]) < 0 && Character.getNumericValue(buf[1]) < 0)
+				if(Character.getType(buf[0]) == 28 && Character.getType(buf[1]) == 28)
+					return new String(Arrays.copyOfRange(buf, 2, length));
+		if(length > 0)
+			if(Character.getNumericValue(buf[0]) < 0)
+				if(Character.getType(buf[0]) == 16)
+					return new String(Arrays.copyOfRange(buf, 1, length));
+		return line;
+    }
+
+}
