@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -52,7 +52,6 @@ package com.openexchange.data.conversion.ical.ical4j;
 import net.fortuna.ical4j.model.Property;
 import net.fortuna.ical4j.model.component.CalendarComponent;
 import net.fortuna.ical4j.model.property.Uid;
-
 import com.openexchange.data.conversion.ical.ICalItem;
 
 /**
@@ -69,11 +68,11 @@ public final class ICal4jItem implements ICalItem {
     }
 
     public String getUID() {
-        Property property = component.getProperty(Uid.UID);
+        final Property property = component.getProperty(Uid.UID);
         return null == property ? null : property.getValue();
     }
 
-    public void setUID(String value) {
+    public void setUID(final String value) {
         if (component.getProperties().contains(Property.UID)) {
             ((Uid) component.getProperty(Property.UID)).setValue(value);
         } else {
