@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,9 +49,7 @@
 package com.openexchange.data.conversion.ical.ical4j.internal;
 
 import java.util.List;
-
 import net.fortuna.ical4j.model.component.CalendarComponent;
-
 import com.openexchange.data.conversion.ical.ConversionError;
 import com.openexchange.data.conversion.ical.ConversionWarning;
 import com.openexchange.groupware.container.CalendarObject;
@@ -80,8 +78,8 @@ public abstract class AbstractVerifyingAttributeConverter<T extends CalendarComp
         }
         this.verifier.verify(index, object, warnings);
     }
-    
-    protected boolean isSet(U calendarObject, int value) {
+
+    protected boolean isSet(final U calendarObject, final int value) {
         return calendarObject.contains(value) && calendarObject.get(value) !=null;
     }
 
