@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -52,8 +52,8 @@ import java.util.List;
 import java.util.TimeZone;
 import net.fortuna.ical4j.model.component.VEvent;
 import net.fortuna.ical4j.model.property.Transp;
-import com.openexchange.data.conversion.ical.ConversionError;
 import com.openexchange.data.conversion.ical.ConversionWarning;
+import com.openexchange.data.conversion.ical.Mode;
 import com.openexchange.data.conversion.ical.ical4j.internal.AbstractVerifyingAttributeConverter;
 import com.openexchange.groupware.container.Appointment;
 import com.openexchange.groupware.contexts.Context;
@@ -62,11 +62,16 @@ import com.openexchange.groupware.contexts.Context;
  * @author Francisco Laguna <francisco.laguna@open-xchange.com>
  */
 public class Transparency extends AbstractVerifyingAttributeConverter<VEvent, Appointment> {
+
+    public Transparency() {
+        super();
+    }
+
     public boolean isSet(final Appointment appointment) {
         return appointment.containsShownAs();
     }
 
-    public void emit(final int index, final Appointment appointment, final VEvent event, final List<ConversionWarning> warnings, final Context ctx, Object... args) throws ConversionError {
+    public void emit(final Mode mode, final int index, final Appointment appointment, final VEvent event, final List<ConversionWarning> warnings, final Context ctx, final Object... args) {
         switch(appointment.getShownAs()) {
             case Appointment.RESERVED :
                 event.getProperties().add(new Transp("OPAQUE"));
@@ -82,7 +87,7 @@ public class Transparency extends AbstractVerifyingAttributeConverter<VEvent, Ap
         return event.getTransparency() != null;
     }
 
-    public void parse(final int index, final VEvent event, final Appointment appointment, final TimeZone timeZone, final Context ctx, final List<ConversionWarning> warnings) throws ConversionError {
+    public void parse(final int index, final VEvent event, final Appointment appointment, final TimeZone timeZone, final Context ctx, final List<ConversionWarning> warnings) {
         final String value = event.getProperty("TRANSP").getValue().toLowerCase();
         if("opaque".equals(value))  {
             appointment.setShownAs(Appointment.RESERVED);
