@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -60,13 +60,14 @@ import net.fortuna.ical4j.model.property.Clazz;
 import com.openexchange.data.conversion.ical.ConversionError;
 import com.openexchange.data.conversion.ical.ConversionWarning;
 import com.openexchange.data.conversion.ical.ConversionWarning.Code;
+import com.openexchange.data.conversion.ical.Mode;
 import com.openexchange.data.conversion.ical.ical4j.internal.AbstractVerifyingAttributeConverter;
 import com.openexchange.groupware.container.CalendarObject;
 import com.openexchange.groupware.contexts.Context;
 
 /**
  * {@link Klass} - Represents the attribute converter for <code>"CLASS"</code> element.
- * 
+ *
  * @author Francisco Laguna <francisco.laguna@open-xchange.com>
  */
 public class Klass<T extends CalendarComponent, U extends CalendarObject> extends AbstractVerifyingAttributeConverter<T, U> {
@@ -88,8 +89,8 @@ public class Klass<T extends CalendarComponent, U extends CalendarObject> extend
     /**
      * {@inheritDoc}
      */
-    public void emit(final int index, final U cObj, final T component, final List<ConversionWarning> warnings,
-            final Context ctx, Object... args) throws ConversionError {
+    public void emit(final Mode mode, final int index, final U cObj, final T component,
+            final List<ConversionWarning> warnings, final Context ctx, final Object... args) throws ConversionError {
         if (cObj.getPrivateFlag()) {
             component.getProperties().add(PRIVATE);
         } else {
@@ -128,10 +129,10 @@ public class Klass<T extends CalendarComponent, U extends CalendarObject> extend
         if (null == str) {
             return true;
         }
-        final char[] chars = str.toCharArray();
+        final int length = str.length();
         boolean empty = true;
-        for (int i = 0; empty && i < chars.length; i++) {
-            empty = Character.isWhitespace(chars[i]);
+        for (int i = 0; empty && i < length; i++) {
+            empty = Character.isWhitespace(str.charAt(i));
         }
         return empty;
     }
