@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -53,17 +53,22 @@ import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.TimeZone;
+import javax.mail.internet.AddressException;
+import javax.mail.internet.IDNA;
 import net.fortuna.ical4j.model.Parameter;
 import net.fortuna.ical4j.model.ParameterList;
 import net.fortuna.ical4j.model.Property;
 import net.fortuna.ical4j.model.PropertyList;
-import net.fortuna.ical4j.model.ResourceList;
+import net.fortuna.ical4j.model.TextList;
 import net.fortuna.ical4j.model.component.CalendarComponent;
+import net.fortuna.ical4j.model.parameter.Cn;
 import net.fortuna.ical4j.model.parameter.CuType;
 import net.fortuna.ical4j.model.parameter.PartStat;
 import net.fortuna.ical4j.model.parameter.Role;
@@ -75,6 +80,7 @@ import org.apache.commons.logging.LogFactory;
 import com.openexchange.data.conversion.ical.ConversionError;
 import com.openexchange.data.conversion.ical.ConversionWarning;
 import com.openexchange.data.conversion.ical.ConversionWarning.Code;
+import com.openexchange.data.conversion.ical.Mode;
 import com.openexchange.data.conversion.ical.ical4j.internal.AbstractVerifyingAttributeConverter;
 import com.openexchange.data.conversion.ical.ical4j.internal.OXResourceResolver;
 import com.openexchange.data.conversion.ical.ical4j.internal.ResourceResolver;
@@ -101,7 +107,7 @@ import com.openexchange.server.ServiceException;
  */
 public class Participants<T extends CalendarComponent, U extends CalendarObject> extends AbstractVerifyingAttributeConverter<T,U> {
 
-    private static Log LOG = LogFactory.getLog(Participants.class);
+    private static Log LOG = com.openexchange.log.Log.valueOf(LogFactory.getLog(Participants.class));
 
     public static UserResolver userResolver = UserResolver.EMPTY;
 
@@ -111,14 +117,33 @@ public class Participants<T extends CalendarComponent, U extends CalendarObject>
         return cObj.containsParticipants();
     }
 
-    public void emit(final int index, final U cObj, final T component, final List<ConversionWarning> warnings, final Context ctx, Object... args) throws ConversionError {
+    public void emit(final Mode mode, final int index, final U cObj, final T component, final List<ConversionWarning> warnings, final Context ctx, final Object... args) throws ConversionError {
         final List<ResourceParticipant> resources = new LinkedList<ResourceParticipant>();
-        for(final Participant p : cObj.getParticipants()) {
+        UserParticipant[] users = cObj.getUsers();
+        ConfirmableParticipant[] confirmations = cObj.getConfirmations();
+        
+        for(Participant p : cObj.getParticipants()) {
             switch(p.getType()) {
                 case Participant.USER:
+                	if (users != null) {
+                		for (UserParticipant up : users) {
+							if (up.getIdentifier() == p.getIdentifier()) {
+								p = up;
+								break;
+							}
+						}
+                	}
                     addUserAttendee(index, (UserParticipant)p, ctx, component, cObj);
                     break;
                 case Participant.EXTERNAL_USER:
+                	if (confirmations != null) {
+                		for (ConfirmableParticipant cp : confirmations) {
+							if (cp.getEmailAddress().equalsIgnoreCase(p.getEmailAddress())) {
+								p = cp;
+								break;
+							}
+						}
+                	}
                     addExternalAttendee((ExternalUserParticipant)p, component);
                     break;
                 case Participant.RESOURCE:
@@ -133,7 +158,7 @@ public class Participants<T extends CalendarComponent, U extends CalendarObject>
 
     private void setResources(final int index, final T component,
         final List<ResourceParticipant> resources, final Context ctx) throws ConversionError {
-        final ResourceList list = new ResourceList();
+        final TextList list = new TextList();
         for (final ResourceParticipant res : resources) {
             String displayName = res.getDisplayName();
             if (null == displayName) {
@@ -155,38 +180,46 @@ public class Participants<T extends CalendarComponent, U extends CalendarObject>
     protected void addExternalAttendee(final ExternalUserParticipant externalUserParticipant, final T component) {
         final Attendee attendee = new Attendee();
         try {
-            attendee.setValue("mailto:" + externalUserParticipant.getEmailAddress());
-            ParameterList parameters = attendee.getParameters();
+            // TODO: Confirmations
+        	final String emailAddress = IDNA.toACE(externalUserParticipant.getEmailAddress());
+            attendee.setValue("mailto:" + emailAddress);
+            final ParameterList parameters = attendee.getParameters();
             parameters.add(CuType.INDIVIDUAL);
             parameters.add(PartStat.NEEDS_ACTION);
             parameters.add(Role.REQ_PARTICIPANT);
             parameters.add(Rsvp.TRUE);
+            parameters.add(new Cn(emailAddress));
             component.getProperties().add(attendee);
         } catch (final URISyntaxException e) {
             LOG.error(e); // Shouldn't happen
+        } catch (AddressException e) {
+            LOG.error(e);
         }
     }
 
-    protected void addUserAttendee(int index, UserParticipant userParticipant, Context ctx, T component, U obj) throws ConversionError {
+    protected void addUserAttendee(final int index, final UserParticipant userParticipant, final Context ctx, final T component, final U obj) throws ConversionError {
         final Attendee attendee = new Attendee();
         try {
             String address;
             //This sets the attendees email-addresses to their DefaultSenderAddress if configured via com.openexchange.notification.fromSource in notification.properties
-            String senderSource = NotificationConfig.getProperty(NotificationProperty.FROM_SOURCE, "primaryMail");
-            if ("defaultSenderAddress".equals(senderSource)) { 
+            final String senderSource = NotificationConfig.getProperty(NotificationProperty.FROM_SOURCE, "primaryMail");
+            if ("defaultSenderAddress".equals(senderSource)) {
                 try {
                     address = UserSettingMailStorage.getInstance().loadUserSettingMail(userParticipant.getIdentifier(), ctx).getSendAddr();
-                } catch (UserConfigurationException e) {
+                } catch (final UserConfigurationException e) {
                     LOG.error(e.getMessage(), e);
                     address = resolveUserMail(index, userParticipant, ctx);
                 }
             } else {
                 address = resolveUserMail(index, userParticipant, ctx);
             }
+            address = IDNA.toACE(address);
             attendee.setValue("mailto:" + address);
-            ParameterList parameters = attendee.getParameters();
+            final ParameterList parameters = attendee.getParameters();
             parameters.add(Role.REQ_PARTICIPANT);
             parameters.add(CuType.INDIVIDUAL);
+            final String displayName = this.resolveUserDisplayName(index, userParticipant, ctx);
+            parameters.add(new Cn(null != displayName && 0 < displayName.length() ? displayName : address));
             switch (userParticipant.getConfirm()) {
             case CalendarObject.ACCEPT:
                 parameters.add(PartStat.ACCEPTED);
@@ -195,7 +228,7 @@ public class Participants<T extends CalendarComponent, U extends CalendarObject>
                 parameters.add(PartStat.DECLINED);
                 break;
             case CalendarObject.TENTATIVE:
-                parameters.add(PartStat.NEEDS_ACTION);
+                parameters.add(PartStat.TENTATIVE);
                 break;
             case CalendarObject.NONE:
             default:
@@ -203,18 +236,31 @@ public class Participants<T extends CalendarComponent, U extends CalendarObject>
             component.getProperties().add(attendee);
         } catch (final URISyntaxException e) {
             LOG.error(e.getMessage(), e); // Shouldn't happen
+        } catch (AddressException e) {
+            LOG.error(e);
         }
     }
 
-    protected String resolveUserMail(int index, UserParticipant userParticipant, Context ctx) throws ConversionError {
+    protected String resolveUserDisplayName(final int index, final UserParticipant userParticipant, final Context ctx) throws ConversionError {
+        try {
+            final User user = userResolver.loadUser(userParticipant.getIdentifier(), ctx);
+            return user.getDisplayName();
+        } catch (final UserException e) {
+            throw new ConversionError(index, e);
+        } catch (final ServiceException e) {
+            throw new ConversionError(index, e);
+        }
+    }
+
+    protected String resolveUserMail(final int index, final UserParticipant userParticipant, final Context ctx) throws ConversionError {
         String address = userParticipant.getEmailAddress();
         if (address == null) {
             try {
                 final User user = userResolver.loadUser(userParticipant.getIdentifier(), ctx);
                 address = user.getMail();
-            } catch (UserException e) {
+            } catch (final UserException e) {
                 throw new ConversionError(index, e);
-            } catch (ServiceException e) {
+            } catch (final ServiceException e) {
                 throw new ConversionError(index, e);
             }
         }
@@ -231,8 +277,8 @@ public class Participants<T extends CalendarComponent, U extends CalendarObject>
         final PropertyList properties = component.getProperties(Property.ATTENDEE);
         final Map<String, ICalParticipant> mails = new HashMap<String, ICalParticipant>();
         final List<String> resourceNames = new LinkedList<String>();
-        
-        String comment = component.getProperty(Property.COMMENT) == null ? null : component.getProperty(Property.COMMENT).getValue();
+
+        final String comment = component.getProperty(Property.COMMENT) == null ? null : component.getProperty(Property.COMMENT).getValue();
 
         for(int i = 0, size = properties.size(); i < size; i++) {
             final Attendee attendee = (Attendee) properties.get(i);
@@ -243,9 +289,9 @@ public class Participants<T extends CalendarComponent, U extends CalendarObject>
                 }
             } else {
                 final URI uri = attendee.getCalAddress();
-                if("mailto".equalsIgnoreCase(uri.getScheme())) {
-                    final String mail = uri.getSchemeSpecificPart();
-                    ICalParticipant icalP = createIcalParticipant(attendee, mail, comment);
+                if(null != uri && "mailto".equalsIgnoreCase(uri.getScheme())) {
+                    final String mail = IDNA.toIDN(uri.getSchemeSpecificPart());
+                    final ICalParticipant icalP = createIcalParticipant(attendee, mail, comment);
                     mails.put(mail, icalP);
                 }
             }
@@ -261,44 +307,58 @@ public class Participants<T extends CalendarComponent, U extends CalendarObject>
         }
 
         for(final User user : users) {
-            UserParticipant up = new UserParticipant(user.getId());
+            final UserParticipant up = new UserParticipant(user.getId());
             ICalParticipant icalP = null;
-            for (String alias: user.getAliases()) {
-                icalP = mails.get(alias);
-                if (icalP != null) {
-                    mails.remove(alias);
+            for (final String alias: getPossibleAddresses(user)) {
+            	String toRemove = null;
+            	for(final String mail: mails.keySet()){
+            		if(mail.equalsIgnoreCase(alias)){
+            			icalP = mails.get(mail);
+            			toRemove = mail;
+            			break;
+            		}
+            	}
+                if (toRemove != null) {
+                    mails.remove(toRemove);
                     break;
                 }
             }
-            if (icalP == null)
-                throw new IllegalStateException("Should not be possible to find a user by it's alias and then be unable to remove that alias from list");
-            if (icalP.message != null)
-                up.setConfirmMessage(icalP.message);
-            if (icalP.status != -1)
-                up.setConfirm(icalP.status);
-            
-            cObj.addParticipant( new UserParticipant(user.getId()) ); //TODO Is that a bug? Which one?
+            if (icalP == null) {
+                LOG.warn("Should not be possible to find a user ("+user.getMail()+") by their alias and then be unable to remove that alias  from list");
+            } else {
+                if (icalP.message != null) {
+                    up.setConfirmMessage(icalP.message);
+                }
+                if (icalP.status != -1) {
+                    up.setConfirm(icalP.status);
+                }
+            }
+
+            cObj.addParticipant(up);
         }
 
-        List<ConfirmableParticipant> confirmableParticipants = new ArrayList<ConfirmableParticipant>();
+        final List<ConfirmableParticipant> confirmableParticipants = new ArrayList<ConfirmableParticipant>();
         for(final String mail : mails.keySet()) {
             final ExternalUserParticipant external = new ExternalUserParticipant(mail);
             external.setDisplayName(null);
 
-            ICalParticipant icalP = mails.get(mail);
-            
-            if (icalP.message != null)
+            final ICalParticipant icalP = mails.get(mail);
+
+            if (icalP.message != null) {
                 external.setMessage(icalP.message);
-            if (icalP.status != -1)
+            }
+            if (icalP.status != -1) {
                 external.setConfirm(icalP.status);
-            
-            if (comment != null)
+            }
+
+            if (comment != null) {
                 external.setMessage(comment);
-            
+            }
+
             cObj.addParticipant(external);
             confirmableParticipants.add(external);
         }
-        
+
         if (confirmableParticipants.size() > 0) {
             cObj.setConfirmations(confirmableParticipants.toArray(new ConfirmableParticipant[confirmableParticipants.size()]));
         }
@@ -330,15 +390,28 @@ public class Participants<T extends CalendarComponent, U extends CalendarObject>
             warnings.add(warning);
         }
     }
-    
+
+    private static Set<String> getPossibleAddresses(User user) {
+        Set<String> possibleAddresses = new HashSet<String>();
+        if (null != user.getMail()) {
+            possibleAddresses.add((user.getMail()));
+        }
+        if (null != user.getAliases()) {
+            for (String alias : user.getAliases()) {
+                possibleAddresses.add(alias);
+            }
+        }
+        return possibleAddresses;
+    }
+
     /**
      * @param attendee
      * @return
      */
-    private ICalParticipant createIcalParticipant(Attendee attendee, String mail, String message) {
-        ICalParticipant retval = new ICalParticipant(mail, -1, message);
+    private ICalParticipant createIcalParticipant(final Attendee attendee, final String mail, final String message) {
+        final ICalParticipant retval = new ICalParticipant(mail, -1, message);
 
-        Parameter parameter = attendee.getParameter(Parameter.PARTSTAT);
+        final Parameter parameter = attendee.getParameter(Parameter.PARTSTAT);
         if (parameter != null) {
             if (parameter.equals(PartStat.ACCEPTED)) {
                 retval.status = CalendarObject.ACCEPT;
@@ -357,8 +430,8 @@ public class Participants<T extends CalendarComponent, U extends CalendarObject>
         public String mail;
         public int status;
         public String message;
-        
-        public ICalParticipant(String mail, int status, String message) {
+
+        public ICalParticipant(final String mail, final int status, final String message) {
             this.mail = mail;
             this.status = status;
             this.message = message;
