@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -53,18 +53,18 @@ import static com.openexchange.data.conversion.ical.ical4j.internal.EmitterTools
 import static com.openexchange.data.conversion.ical.ical4j.internal.EmitterTools.toDateTime;
 import static com.openexchange.data.conversion.ical.ical4j.internal.ParserTools.isDateTime;
 import static com.openexchange.data.conversion.ical.ical4j.internal.ParserTools.parseDateConsideringDateType;
-
 import java.util.Calendar;
 import java.util.Date;
 import java.util.GregorianCalendar;
 import java.util.List;
 import java.util.TimeZone;
-
 import net.fortuna.ical4j.model.component.CalendarComponent;
+import net.fortuna.ical4j.model.property.DtEnd;
 import net.fortuna.ical4j.model.property.DtStart;
-
 import com.openexchange.data.conversion.ical.ConversionWarning;
+import com.openexchange.data.conversion.ical.Mode;
 import com.openexchange.data.conversion.ical.ical4j.internal.AbstractVerifyingAttributeConverter;
+import com.openexchange.data.conversion.ical.ical4j.internal.EmitterTools;
 import com.openexchange.groupware.container.Appointment;
 import com.openexchange.groupware.container.CalendarObject;
 import com.openexchange.groupware.contexts.Context;
@@ -75,19 +75,14 @@ import com.openexchange.groupware.contexts.Context;
  */
 public final class Start<T extends CalendarComponent, U extends CalendarObject> extends AbstractVerifyingAttributeConverter<T,U> {
 
-    /**
-     * Default constructor.
-     */
     public Start() {
         super();
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public void emit(final int index, final U calendar, final T component, final List<ConversionWarning> warnings, final Context ctx, Object... args) {
+    public void emit(final Mode mode, final int index, final U calendar, final T component, final List<ConversionWarning> warnings, final Context ctx, final Object... args) {
         final DtStart start = new DtStart();
-        final net.fortuna.ical4j.model.Date date = (needsDate(calendar)) ? toDate(calendar.getStartDate()) : toDateTime(calendar.getStartDate());
+        String tz = EmitterTools.extractTimezoneIfPossible(calendar);
+        final net.fortuna.ical4j.model.Date date = (needsDate(calendar)) ? toDate(calendar.getStartDate()) : toDateTime(mode.getZoneInfo(), calendar.getStartDate(),tz);
         start.setDate(date);
         component.getProperties().add(start);
     }
@@ -96,16 +91,10 @@ public final class Start<T extends CalendarComponent, U extends CalendarObject>
         return Appointment.class.isAssignableFrom(calendar.getClass()) && ((Appointment)calendar).getFullTime();
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public boolean hasProperty(final T component) {
         return null != component.getProperty(DtStart.DTSTART);
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public boolean isSet(final U calendar) {
         return calendar.containsStartDate();
     }
@@ -119,30 +108,33 @@ public final class Start<T extends CalendarComponent, U extends CalendarObject>
         final boolean isDateTime = isDateTime(component, dtStart);
         final TimeZone UTC = TimeZone.getTimeZone("UTC");
         final Date start = parseDateConsideringDateType(component, dtStart, timeZone);
-        
+
         calendar.setStartDate(start);
-        // If an end is specified end date will be overwritten.
-        if (isDateTime) {
-            /* RFC 2445 4.6.1:
-             * For cases where a "VEVENT" calendar component specifies a "DTSTART"
-             * property with a DATE-TIME data type but no "DTEND" property, the
-             * event ends on the same calendar date and time of day specified by
-             * the "DTSTART" property.
-             */
-            calendar.setEndDate(start);
-        } else {
-            // Only the date is specified. Then we have to set the end to at
-            // least 1 day later. Will be overwritten if DTEND is specified.
-            final Calendar calendarUTC = new GregorianCalendar();
-            calendarUTC.setTimeZone(UTC);
-            calendarUTC.setTime(start);
-            calendarUTC.add(Calendar.DATE, 1);
-            calendar.setEndDate(calendarUTC.getTime());
-            // Special flag for appointments.
-            if (calendar instanceof Appointment) {
-                final Appointment appointment = (Appointment) calendar;
-                appointment.setFullTime(true);
+        if (component.getProperty(DtEnd.DTEND) == null && calendar instanceof Appointment) {
+            // If an end is specified end date will be overwritten.
+            if (isDateTime) {
+                /* RFC 2445 4.6.1:
+                 * For cases where a "VEVENT" calendar component specifies a "DTSTART"
+                 * property with a DATE-TIME data type but no "DTEND" property, the
+                 * event ends on the same calendar date and time of day specified by
+                 * the "DTSTART" property.
+                 */
+                calendar.setEndDate(start);
+            } else {
+                // Only the date is specified. Then we have to set the end to at
+                // least 1 day later. Will be overwritten if DTEND is specified.
+                final Calendar calendarUTC = new GregorianCalendar();
+                calendarUTC.setTimeZone(UTC);
+                calendarUTC.setTime(start);
+                calendarUTC.add(Calendar.DATE, 1);
+                calendar.setEndDate(calendarUTC.getTime());
+                // Special flag for appointments.
             }
         }
+        
+        if (!isDateTime && calendar instanceof Appointment) {
+            final Appointment appointment = (Appointment) calendar;
+            appointment.setFullTime(true);
+        }
     }
 }
