@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -52,12 +52,10 @@ package com.openexchange.data.conversion.ical.ical4j.internal.task;
 import java.util.Date;
 import java.util.List;
 import java.util.TimeZone;
-
 import net.fortuna.ical4j.model.component.VToDo;
 import net.fortuna.ical4j.model.property.Completed;
-
-import com.openexchange.data.conversion.ical.ConversionError;
 import com.openexchange.data.conversion.ical.ConversionWarning;
+import com.openexchange.data.conversion.ical.Mode;
 import com.openexchange.data.conversion.ical.ical4j.internal.AbstractVerifyingAttributeConverter;
 import com.openexchange.data.conversion.ical.ical4j.internal.EmitterTools;
 import com.openexchange.data.conversion.ical.ical4j.internal.ParserTools;
@@ -70,41 +68,24 @@ import com.openexchange.groupware.tasks.Task;
  */
 public class DateCompleted extends AbstractVerifyingAttributeConverter<VToDo, Task> {
 
-    /**
-     * Default constructor.
-     */
     public DateCompleted() {
         super();
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public boolean isSet(final Task task) {
         return task.containsDateCompleted();
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public void emit(final int index, final Task task, final VToDo vToDo,
-                     final List<ConversionWarning> warnings, final Context ctx, Object... args) throws ConversionError {
+    public void emit(final Mode mode, final int index, final Task task, final VToDo vToDo, final List<ConversionWarning> warnings, final Context ctx, final Object... args) {
         final Completed completed = new Completed(EmitterTools.toDateTime(task.getDateCompleted()));
         vToDo.getProperties().add(completed);
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public boolean hasProperty(final VToDo vToDo) {
         return vToDo.getDateCompleted() != null;
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public void parse(final int index, final VToDo vToDo, final Task task, final TimeZone timeZone,
-        final Context ctx, final List<ConversionWarning> warnings) throws ConversionError {
+    public void parse(final int index, final VToDo vToDo, final Task task, final TimeZone timeZone, final Context ctx, final List<ConversionWarning> warnings) {
         final Completed completed =  vToDo.getDateCompleted();
         final Date completedDate = ParserTools.toDateConsideringDateType(completed,timeZone);
         task.setDateCompleted(completedDate);
