@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,16 +49,17 @@
 
 package com.openexchange.data.conversion.ical.ical4j.internal.task;
 
+import static com.openexchange.data.conversion.ical.ical4j.internal.EmitterTools.toDate;
+
 import java.util.Date;
 import java.util.List;
 import java.util.TimeZone;
-
 import net.fortuna.ical4j.model.component.VToDo;
 import net.fortuna.ical4j.model.property.Due;
-
-import com.openexchange.data.conversion.ical.ConversionError;
 import com.openexchange.data.conversion.ical.ConversionWarning;
+import com.openexchange.data.conversion.ical.Mode;
 import com.openexchange.data.conversion.ical.ical4j.internal.AbstractVerifyingAttributeConverter;
+import com.openexchange.data.conversion.ical.ical4j.internal.EmitterTools;
 import com.openexchange.data.conversion.ical.ical4j.internal.ParserTools;
 import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.tasks.Task;
@@ -69,32 +70,25 @@ import com.openexchange.groupware.tasks.Task;
  */
 public class DueDate extends AbstractVerifyingAttributeConverter<VToDo, Task> {
 
-    /**
-     * Default constructor.
-     */
     public DueDate() {
         super();
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public boolean isSet(final Task task) {
-        return false;  // EndDate only
+        return task.containsEndDate();
     }
 
-    public void emit(final int index, final Task task, final VToDo vToDo, final List<ConversionWarning> warnings, final Context ctx, Object... args) throws ConversionError {
-       return; // EndDate only
+    public void emit(final Mode mode, final int index, final Task task, final VToDo vToDo, final List<ConversionWarning> warnings, final Context ctx, final Object... args) {
+        String tz = EmitterTools.extractTimezoneIfPossible(task);
+        net.fortuna.ical4j.model.Date date = toDate(task.getEndDate(), tz);
+        vToDo.getProperties().add(new Due(date));
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public boolean hasProperty(final VToDo vToDo) {
         return null != vToDo.getDue();
     }
 
-    public void parse(final int index, final VToDo vToDo, final Task task, final TimeZone timeZone, final Context ctx, final List<ConversionWarning> warnings) throws ConversionError {
+    public void parse(final int index, final VToDo vToDo, final Task task, final TimeZone timeZone, final Context ctx, final List<ConversionWarning> warnings) {
         if(task.containsEndDate()) {
             return;
         }
