@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -53,16 +53,19 @@ import org.osgi.framework.BundleActivator;
 import org.osgi.framework.BundleContext;
 import org.osgi.framework.ServiceRegistration;
 import org.osgi.util.tracker.ServiceTracker;
-
 import com.openexchange.data.conversion.ical.ICalEmitter;
 import com.openexchange.data.conversion.ical.ICalParser;
 import com.openexchange.data.conversion.ical.ical4j.ICal4JEmitter;
+import com.openexchange.data.conversion.ical.ical4j.ICal4JITipEmitter;
+import com.openexchange.data.conversion.ical.ical4j.ICal4JITipParser;
 import com.openexchange.data.conversion.ical.ical4j.ICal4JParser;
 import com.openexchange.data.conversion.ical.ical4j.internal.OXResourceResolver;
 import com.openexchange.data.conversion.ical.ical4j.internal.OXUserResolver;
 import com.openexchange.data.conversion.ical.ical4j.internal.UserResolver;
 import com.openexchange.data.conversion.ical.ical4j.internal.calendar.CreatedBy;
 import com.openexchange.data.conversion.ical.ical4j.internal.calendar.Participants;
+import com.openexchange.data.conversion.ical.itip.ITipEmitter;
+import com.openexchange.data.conversion.ical.itip.ITipParser;
 import com.openexchange.groupware.calendar.CalendarCollectionService;
 import com.openexchange.resource.ResourceService;
 import com.openexchange.user.UserService;
@@ -95,6 +98,10 @@ public class Activator implements BundleActivator {
      */
     private ServiceRegistration emitterRegistration;
 
+    private ServiceRegistration itipParserRegistration;
+
+    private ServiceRegistration itipEmitterRegistration;
+
     /**
      * {@inheritDoc}
      */
@@ -115,6 +122,8 @@ public class Activator implements BundleActivator {
 
         parserRegistration = context.registerService(ICalParser.class.getName(), new ICal4JParser(), null);
         emitterRegistration = context.registerService(ICalEmitter.class.getName(), new ICal4JEmitter(), null);
+        itipParserRegistration = context.registerService(ITipParser.class.getName(), new ICal4JITipParser(), null);
+        itipEmitterRegistration = context.registerService(ITipEmitter.class.getName(), new ICal4JITipEmitter(), null);
     }
 
     /**
@@ -123,6 +132,8 @@ public class Activator implements BundleActivator {
     public void stop(final BundleContext context) throws Exception {
         emitterRegistration.unregister();
         parserRegistration.unregister();
+        itipParserRegistration.unregister();
+        itipEmitterRegistration.unregister();
         calendarTracker.close();
         resourceTracker.close();
         CreatedBy.userResolver = UserResolver.EMPTY;
